///****************************************************************************
//
//	Copyright 2011 by WinSystems Inc.
//
//	Permission is hereby granted to the purchaser of WinSystems CPU products
//  to distribute any binary file or files compiled using this source code
//	directly or in any work derived by the user from this file. In no case
//	may the source code, original or derived from this file, be distributed
//	to any third party except by explicit permission of WinSystems. This file
//	is distributed on an "as-is" basis and no warranty as to performance or
//	fitness of purposes is expressed or implied. In no case shall WinSystems
//	be liable for any direct or indirect loss or damage, real or consequential
//	resulting from the usage of this source code. It is the user's sole
//	responsibility to determine fitness for any considered purpose.
//
///****************************************************************************
//
//	Name	 : buffered.cpp
//
//	Project	 : PCM-MIO-G Sample Application
//
//	Author	 : Paul DeMetrotion
//
///****************************************************************************
//
//	  Date		Revision	                Description
//	--------	--------	---------------------------------------------
//	 11/8/11	  1.0		  Original Release
//
///****************************************************************************

#include "stdafx.h"
#include "..\mio_io\mio_io.h"
#include <stdio.h>
#include <conio.h>

#define DEVNUM 0

UCHAR to_do_channels[2001];
USHORT values[2000];

int _tmain(int argc, _TCHAR* argv[])
{
	int channel = 0;
	USHORT result;
	float current;
	ULONG count = 0;
	int x;

	// Set up all 16 channels for the +/- 10 Volt range
	for(channel =0; channel < 16; channel ++)
	{
		adc_set_channel_mode(DEVNUM,channel,ADC_SINGLE_ENDED,ADC_BIPOLAR,ADC_TOP_10V);
		
		if(mio_error_code)
		{
			printf("\nError occured - %s\n",mio_error_string);
			exit(1);
		}
	}

	// We'll fill the to_do list with the four different channels 500
	// each successively.
	for(x=0; x < 500; x++)
	{
		to_do_channels[x] = 0;
		to_do_channels[x+500] = 1;
		to_do_channels[x+1000] = 2;
		to_do_channels[x+1500] = 3;

	}

	// Load the "terminator into the last position 
	to_do_channels[2000] = 0xff;

	// We'll keep going until a key is pressed
	while(!_kbhit())
	{
		// Start up the conversions. This function returns when all 2000 of
	    // our conversions are complete.
		adc_buffered_channel_conversions(DEVNUM,to_do_channels,values);
		
		count += 2000;

		if(mio_error_code)
		{
			printf("\nError occured - %s\n",mio_error_string);
			exit(1);
		}

		// We'll extract our data from the "values" array. In order to make the
		// display more readable, we'll take a value from each channel display them,
		// and move to the next result.
		for(x=0; x < 500; x++)
		{
			printf("%08ld  ",count);
			
			// Get the raw data
			result = values[x];

			// Convert to voltage
			current = adc_convert_to_volts(DEVNUM, 0, result);

			// Display the value
			printf("CH0 %9.5f ",current);

			// Repeat for channels 1- 3 
			result = values[x+500];
			current = adc_convert_to_volts(DEVNUM, 1, result);
			printf("CH1 %9.5f ",current);

			result = values[x+1000];
			current = adc_convert_to_volts(DEVNUM, 2, result);
			printf("CH2 %9.5f ",current);
			
			result = values[x+1500];
			current = adc_convert_to_volts(DEVNUM, 3, result);
			printf("CH3 %9.5f ",current);
			printf("\r");
		}
	}

	_getch();

	return 0;
}
