///****************************************************************************
//
//	Copyright 2011 by WinSystems Inc.
//
//	Permission is hereby granted to the purchaser of WinSystems CPU products
//  to distribute any binary file or files compiled using this source code
//	directly or in any work derived by the user from this file. In no case
//	may the source code, original or derived from this file, be distributed
//	to any third party except by explicit permission of WinSystems. This file
//	is distributed on an "as-is" basis and no warranty as to performance or
//	fitness of purposes is expressed or implied. In no case shall WinSystems
//	be liable for any direct or indirect loss or damage, real or consequential
//	resulting from the usage of this source code. It is the user's sole
//	responsibility to determine fitness for any considered purpose.
//
///****************************************************************************
//
//	Name	 : repeat.cpp
//
//	Project	 : PCM-MIO-G Sample Application
//
//	Author	 : Paul DeMetrotion
//
///****************************************************************************
//
//	  Date		Revision	                Description
//	--------	--------	---------------------------------------------
//	 11/8/11	  1.0		  Original Release
//
///****************************************************************************

#include "stdafx.h"
#include "..\mio_io\mio_io.h"
#include <stdio.h>
#include <conio.h>

#define DEVNUM 0

// This array will store the results of 2000 conversions

USHORT values[2000];
ULONG count;

HANDLE my_event;
HANDLE ThreadID;
DWORD WINAPI MyThread(LPVOID junkola);

int _tmain(int argc, _TCHAR* argv[])
{
DWORD junk;
int device = DEVNUM;
int channel = 0;
USHORT result;
float min,max,current;
int x,c;

	// We'll keep track of the minimum and maximum voltage values
	// we see on a channel as well as the count of conversions
	// completed.

	max = -10.0;
	min	= 10.0;
	count = 0;

	// The default channel is 0 but an alternate channel can be specified
	// as a command line argument.

	if(argc > 1)
	{
		device = atoi(argv[1]);
		channel = atoi(argv[2]);
	}

	//  Check device number for validity

	if(device < 0 || device > 3)
	{
		printf("Device number must be between 0 and 3 - Aborting\n");
		exit(0);
	}

	// Check for a valid channel number. Abort if bad

	if(channel <0 || channel > 15)
	{
		printf("Channel numbers must be between 0 and 15 - Aborting\n");
		exit(0);
	}

	// This call sets the mode for the specified channel. We are going to
	// set up for single-ended +/- 10 V range. That way any legal input 
	// can be accomodated.

	adc_set_channel_mode(device,channel,ADC_SINGLE_ENDED,ADC_BIPOLAR,ADC_TOP_10V);
	if(mio_error_code)
	{
		printf("\nError occured - %s\n",mio_error_string);
		exit(1);
	}

	enable_adc_interrupt(device,0);
	enable_adc_interrupt(device,1);

	if(mio_error_code)
	{
		printf("\nError occured - %s\n",mio_error_string);
		exit(1);
	}

	my_event = CreateEvent(NULL,FALSE,FALSE,NULL);

	set_adc_event(device, 0, my_event);
	set_adc_event(device, 1, my_event);


	ThreadID = CreateThread(NULL,0,MyThread,NULL,0,&junk);

	while(1)
	{
			// We'll keep running until a recognized key is pressed 
		if(_kbhit())
		{
				c = _getch();

				// The 'C' key clears the min/max and count values

				if(c== 'c' || c == 'C')
				{
					count = 0;
					min = 10.0;
					max = -10.0;
				}

				// The 'N' key moves to the next channel, wrapping from 15
				// back to 0 when appropriate.

				else if(c == 'n' || c == 'N')
				{
					printf("\n");
					channel++;
					if(channel > 15)
						channel = 0;

					// When we change channels we need to make sure we set
					// the channel's mode to a valid range.

					adc_set_channel_mode(device,channel,ADC_SINGLE_ENDED,ADC_BIPOLAR,ADC_TOP_10V);
					if(mio_error_code)
					{
						printf("\nError occured - %s\n",mio_error_string);
						exit(1);
					}

					// A new channel also clears the count and min/max values.

					count = 0;
					min = 10.0;
					max = -10.0;
				}
				else
				{
					set_adc_event(device,0,NULL);
					set_adc_event(device,1,NULL);
					disable_adc_interrupt(device,0);
					disable_adc_interrupt(device,1);
					exit(0);
				}
		}

		// Finally the real thing. This function-call results in 2000
		// conversions on the specified channel with the results going into
		// a buffer called "values".

		adc_convert_single_repeated(device,channel,2000,values);
		if(mio_error_code)
		{
			printf("\nError occured - %s\n",mio_error_string);
			exit(1);
		}

		// Bump up the count

;		count += 2000;

		// Now we'll read out the 2000 conversion values. Convert them to 
		// floating point voltages and set the min and max values as appropriate
		
		for(x=0; x<2000; x++)
		{
			result = values[x];
			
			current = adc_convert_to_volts(device,channel,result);

			// Check and load the min/max values as needed

			if(current < min)
				min = current;

			if(current > max)
				max = current;

			// Print the values

			printf("CH %02d %09ld %9.5f Min =%9.5f Max =%9.5f\r",channel,count,current,min,max);
		}
	}


	return 0;
}

// This thread awakens with each ADC interrupt. It's only purpose in this sample
// program is to count the number of ADC interrupts that have occured.

DWORD WINAPI MyThread(LPVOID junkola)
{
	while(1)
	{
		WaitForSingleObject(my_event,INFINITE);
		++count;
	}
}
