///****************************************************************************
//
//	Copyright 2011 by WinSystems Inc.
//
//	Permission is hereby granted to the purchaser of WinSystems CPU products
//  to distribute any binary file or files compiled using this source code
//	directly or in any work derived by the user from this file. In no case
//	may the source code, original or derived from this file, be distributed
//	to any third party except by explicit permission of WinSystems. This file
//	is distributed on an "as-is" basis and no warranty as to performance or
//	fitness of purposes is expressed or implied. In no case shall WinSystems
//	be liable for any direct or indirect loss or damage, real or consequential
//	resulting from the usage of this source code. It is the user's sole
//	responsibility to determine fitness for any considered purpose.
//
///****************************************************************************
//
//	Name	 : ints.cpp
//
//	Project	 : PCM-MIO-G Sample Application
//
//	Author	 : Paul DeMetrotion
//
///****************************************************************************
//
//	  Date		Revision	                Description
//	--------	--------	---------------------------------------------
//	 11/8/11	  1.0		  Original Release
//
///****************************************************************************

#include "stdafx.h"
#include "..\mio_io\mio_io.h"
#include <stdio.h>
#include <conio.h>

#define DEVNUM 0

USHORT counts[25];
void display_count(void);

// My event will be the handle to the event that's passed down to the driver
// The driver will signal the event when the interrupt occurs.
HANDLE my_event;

// A thread ID returned by Windows on thread creation. We don't use
// it in this poregram.
HANDLE ThreadID;

// Function prototype for our interrupt handling thread
DWORD WINAPI MyThread(LPVOID junkola);

int _tmain(int argc, _TCHAR* argv[])
{
	DWORD junk;
	int x;

	printf("Enabling Bit sense on 1 - 24 - Press key to exit\n\n");

	for(x=1; x<=24; x++)
	{
		// Clear out the count values
		counts[x] = 0;

		// Enable falling edge event sensing on all 24 lines
		dio_enable_bit_int(DEVNUM,x,FALLING);

		// Check for error
		if(mio_error_code)
		{
			printf("\n%s\n",mio_error_string);
			exit(1);
		}
	}

	// This call enables interrupts on the DIO section of the PCM-MIO board
	enable_dio_interrupt(DEVNUM);
	
	// Check for error
	if(mio_error_code)
	{
		printf("\n%s\n",mio_error_string);
		exit(1);
	}
	
	// Create an event that we can share with the kernel driver
	my_event = CreateEvent(NULL,FALSE,FALSE,NULL);

	// Tell the driver what the event is and what section 
	// it interrupt it applies to (DIO)
	set_dio_event(DEVNUM,my_event);

	// Check for error
	if(mio_error_code)
	{
		printf("\n%s\n",mio_error_string);
		exit(1);
	}

	// Create our interrupt thread.
	ThreadID = CreateThread(NULL,0,MyThread,NULL,0,&junk);

	// Put a header on the screen
	for(x=1; x<=24; x++)
		printf("%2d ",x);

	printf("\n");

	while(!_kbhit())
	{
		// The interrupt thread will receive notification from the driver
		// When the event occurs. We have nothing left to do here but display
		// the results
		display_count();
	}

	_getch();

	// We'll detach our event handle from the driver before exiting
	set_dio_event(DEVNUM,NULL);

	if(mio_error_code)
	{
		printf("\n%s\n",mio_error_string);
		exit(1);
	}

	// We'll also turn off the DIO interrupts before we leave
	disable_dio_interrupt(DEVNUM);

	if(mio_error_code)
	{
		printf("\n%s\n",mio_error_string);
		exit(1);
	}

	return 0;
}

void display_count(void)
{
	int x;

	for(x=1; x<25; x++)
		printf("%2d ",counts[x]);

	printf("\r");
}

// This function becomes a separate thread running concurrently with the main program
// It serves the same purpose as an interrupt service routine (ISR). It goes to sleep
// consuming no processor time until awakened by the kernel driver by signaling the
// event the thread is waiting on. Once awakened the thread retreives all pending
// event sense interrupts, counts them up, and clears them. When there are no more
// events pending, it goes back to sleep until the next time the kernel driver
// signals.
DWORD WINAPI MyThread(LPVOID junkola)
{
	int c;

	while(1)
	{
		// We'll clear out any pending ones when we first start
		while(c = dio_get_int(DEVNUM))
		{
			// Tally the count for each bit
			counts[c] += 1;

			// Due to extremely limited space on an 80 column screen
			// We'll not allow counts to reach 100 as everything would
			// scroll off the screen
			if(counts[c] > 99)
				counts[c] = 0;

			// We also must clear the interrupt so that it's re-armed
			dio_clr_int(DEVNUM,c);
		}

		// Wait here forever (sleeping) until the kernel triggers my_event
		WaitForSingleObject(my_event, INFINITE);

		// Now that I'm awake poll for ALL pending interrupts from the chip
	}

	// This thread does not ever exit. It is killed by Windows when the parent
	// program exits.
}
