///****************************************************************************
//
//	Copyright 2011 by WinSystems Inc.
//
//	Permission is hereby granted to the purchaser of WinSystems CPU products
//  to distribute any binary file or files compiled using this source code
//	directly or in any work derived by the user from this file. In no case
//	may the source code, original or derived from this file, be distributed
//	to any third party except by explicit permission of WinSystems. This file
//	is distributed on an "as-is" basis and no warranty as to performance or
//	fitness of purposes is expressed or implied. In no case shall WinSystems
//	be liable for any direct or indirect loss or damage, real or consequential
//	resulting from the usage of this source code. It is the user's sole
//	responsibility to determine fitness for any considered purpose.
//
///****************************************************************************
//
//	Name	 : poll.cpp
//
//	Project	 : PCM-MIO-G Sample Application
//
//	Author	 : Paul DeMetrotion
//
///****************************************************************************
//
//	  Date		Revision	                Description
//	--------	--------	---------------------------------------------
//	 11/8/11	  1.0		  Original Release
//
///****************************************************************************

#include "stdafx.h"
#include "..\mio_io\mio_io.h"
#include <stdio.h>
#include <conio.h>

#define DEVNUM 0

USHORT counts[25];
void display_count(void);

int _tmain(int argc, _TCHAR* argv[])
{
	int x;

	printf("Enabling Bit sense on 1 - 24 - Press key to exit\n\n");

	for(x=1; x<=24; x++)
	{
		// Clear out the count values
		counts[x] = 0;

		// Enable falling edge event sensing on all 24 lines
		dio_enable_bit_int(DEVNUM, x,FALLING);

		// Check for error
		if(mio_error_code)
		{
			printf("\n%s\n",mio_error_string);
			exit(1);
		}
	}

	// Put a header on the screen
	for(x=1; x<=24; x++)
		printf("%2d ",x);

	printf("\n");

	// Display the initial counts
	display_count();

	while(!_kbhit())
	{
		// Check for any events that have occured, non-zero is a valid event
		x = dio_get_int(DEVNUM);

		// Check for error
		if(mio_error_code)
		{
			printf("\n%s\n",mio_error_string);
			exit(2);
		}

		// If we have an event, update its count and refresh the screen
		if(x)
		{
			counts[x] += 1;

			// Due to extremely limited space on an 80 column screen
			// We'll not allow counts above 99 as everything would
			// scroll off the screen
			if(counts[x] > 99)
				counts[x] = 0;

			// We also must clear the interrupt so that it's re-armed
			dio_clr_int(DEVNUM, x);

			// Check for error
			if(mio_error_code)
			{
				printf("\n%s\n",mio_error_string);
				exit(2);
			}

			// Update the counts on the screen
			display_count();
		}
	}

	_getch();

	return 0;
}

void display_count(void)
{
	int x;

	for(x=1; x<25; x++)
		printf("%2d ",counts[x]);

	printf("\r");
}
